@@ -1,4 +1,3 @@
-import sys
 try:
     from __builtin__ import all as _all, any as _any
 except ImportError:
@@ -6,16 +5,15 @@ except ImportError:
 from operator import itemgetter, methodcaller
 from collections import Mapping, Set, Iterable, Iterator, defaultdict
 from itertools import chain, tee
-from functools import reduce
 
-from .cross import basestring, xrange, izip, map, filter, imap, ifilter, PY2
+from .cross import basestring, xrange, izip, map, filter, imap, PY2
 from .primitives import EMPTY
 from .funcs import identity, partial, compose, complement
 from .funcmakers import wrap_mapper, wrap_selector
 from .seqs import take, ximap, ifilter
 
 
-__all__ = ['empty', 'iteritems',
+__all__ = ['empty', 'iteritems', 'itervalues',
            'join', 'merge', 'join_with', 'merge_with',
            'walk', 'walk_keys', 'walk_values', 'select', 'select_keys', 'select_values', 'compact',
            'is_distinct', 'all', 'any', 'none', 'one', 'some',
@@ -42,10 +40,16 @@ def empty(coll):
 if PY2:
     def iteritems(coll):
         return coll.iteritems() if hasattr(coll, 'iteritems') else coll
+
+    def itervalues(coll):
+        return coll.itervalues() if hasattr(coll, 'itervalues') else coll
 else:
     def iteritems(coll):
         return coll.items() if hasattr(coll, 'items') else coll
 
+    def itervalues(coll):
+        return coll.values() if hasattr(coll, 'values') else coll
+
 
 def join(colls):
     colls, colls_copy = tee(colls)
@@ -68,8 +72,9 @@ def join(colls):
     elif isinstance(dest, (Iterator, xrange)):
         return chain.from_iterable(colls)
     elif isinstance(dest, Iterable):
-        return cls(chain.from_iterable(colls)) # could be reduce(concat, ...)
-                                               # more effective for low count
+        # NOTE: this could be reduce(concat, ...),
+        #       more effective for low count
+        return cls(chain.from_iterable(colls))
     else:
         raise TypeError("Don't know how to join %s" % cls.__name__)
 
