@@ -17,3 +17,8 @@ except ImportError:
 
     def filter(f, seq):
         return list(_filter(f, seq))
+
+
+import sys
+PY2 = sys.version_info[0] == 2
+PY3 = sys.version_info[0] == 3
