@@ -1,4 +1,4 @@
-import sys
+from inspect import getargspec, formatargspec, ArgSpec
 from functools import wraps
 from operator import itemgetter
 from collections import Mapping, Set
@@ -38,7 +38,30 @@ def _wrap_higher_order(func, test):
     #       map(None, ...) is much faster than map(identity, ...),
     #       also map(None, ...) works as zip() for multiple seqs
     builtin = PY2 and func in set([map, filter, imap, ifilter, ifilterfalse])
-    return wraps(func)(lambda f, *seqs: func(make_func(f, builtin=builtin, test=test), *seqs))
+
+    # We are going to construct function using eval to preserve signature.
+    # So we need to inspect it first.
+    try:
+        spec = getargspec(func)
+    except TypeError:
+        spec = ArgSpec(('f',), 'seqs', None, None)
+    # HACK: due to bug in python 3.4 - http://bugs.python.org/issue22203
+    if not spec.args:
+        spec = ArgSpec(('f',), 'seqs', None, None)
+
+    # Slicing with [1:-1] to get rid of parentheses
+    spec_str = formatargspec(*spec)[1:-1]
+    rest = ArgSpec(spec.args[1:], *spec[1:])
+    rest_str = formatargspec(*rest)[1:-1]
+
+    # We use nested lambda to make func and make_func locals which are faster
+    func_str = "lambda __func, __make_func: " \
+               "lambda {spec}: __func(__make_func({f}, {builtin}, {test}), {rest})" \
+               .format(spec=spec_str, f=spec.args[0], rest=rest_str,
+                       builtin=builtin, test=test)
+
+    wrapper = eval(func_str, {}, {})(func, make_func)
+    return wraps(func)(wrapper)
 
 def wrap_mapper(func):
     return _wrap_higher_order(func, test=False)
