@@ -1,4 +1,3 @@
-import sys
 from operator import add
 from itertools import islice, chain, tee, dropwhile, takewhile, groupby
 from collections import defaultdict, deque, Sequence
@@ -55,7 +54,7 @@ def nth(n, seq):
     except IndexError:
         return None
     except TypeError:
-        return next(islice(seq, n, n+1), None)
+        return next(islice(seq, n, None), None)
 
 def last(seq):
     try:
@@ -64,7 +63,7 @@ def last(seq):
         return None
     except TypeError:
         item = None
-        for item in iter(seq):
+        for item in seq:
             pass
         return item
 
@@ -85,8 +84,8 @@ def butlast(seq):
 def ilen(seq):
     """
     Consume an iterable not reading it into memory; return the number of items.
-    Implementation borrowed from http://stackoverflow.com/a/15112059/753382
     """
+    # NOTE: implementation borrowed from http://stackoverflow.com/a/15112059/753382
     counter = count()
     deque(izip(seq, counter), maxlen=0)  # (consume at C speed)
     return next(counter)
